@@ -5,7 +5,6 @@ use pyo3::exceptions::{PyTypeError, PyValueError};
 use pyo3::ffi;
 use pyo3::prelude::*;
 use pyo3::types::{PyBool, PyDict, PyList, PyString};
-use pyo3::ToPyObject;
 
 use smallvec::SmallVec;
 
@@ -45,7 +44,7 @@ impl PythonParse {
     /// # Returns
     ///
     /// A [PyObject](https://docs.rs/pyo3/latest/pyo3/type.PyObject.html) representing the parsed JSON value.
-    pub fn python_parse<'py>(self, py: Python<'py>, json_data: &[u8]) -> JsonResult<Bound<'py, PyAny>> {
+    pub fn python_parse<'py>(&self, py: Python<'py>, json_data: &[u8]) -> JsonResult<Bound<'py, PyAny>> {
         macro_rules! ppp {
             ($string_cache:ident, $key_check:ident, $parse_number:ident) => {
                 PythonParser::<$string_cache, $key_check, $parse_number>::parse(
@@ -93,8 +92,8 @@ struct PythonParser<'j, StringCache, KeyCheck, ParseNumber> {
     partial_mode: PartialMode,
 }
 
-impl<'j, StringCache: StringMaybeCache, KeyCheck: MaybeKeyCheck, ParseNumber: MaybeParseNumber>
-    PythonParser<'j, StringCache, KeyCheck, ParseNumber>
+impl<StringCache: StringMaybeCache, KeyCheck: MaybeKeyCheck, ParseNumber: MaybeParseNumber>
+    PythonParser<'_, StringCache, KeyCheck, ParseNumber>
 {
     fn parse<'py>(
         py: Python<'py>,
@@ -129,38 +128,40 @@ impl<'j, StringCache: StringMaybeCache, KeyCheck: MaybeKeyCheck, ParseNumber: Ma
             }
             Peek::True => {
                 self.parser.consume_true()?;
-                Ok(true.to_object(py).into_bound(py))
+                Ok(PyBool::new(py, true).to_owned().into_any())
             }
             Peek::False => {
                 self.parser.consume_false()?;
-                Ok(false.to_object(py).into_bound(py))
+                Ok(PyBool::new(py, false).to_owned().into_any())
             }
             Peek::String => {
                 let s = self
                     .parser
                     .consume_string::<StringDecoder>(&mut self.tape, self.partial_mode.allow_trailing_str())?;
-                Ok(StringCache::get_value(py, s.as_str(), s.ascii_only()).into_any())
+                Ok(StringCache::get_value(py, s).into_any())
             }
             Peek::Array => {
                 let peek_first = match self.parser.array_first() {
                     Ok(Some(peek)) => peek,
-                    Err(e) if !self._allow_partial_err(&e) => return Err(e),
-                    Ok(None) | Err(_) => return Ok(PyList::empty_bound(py).into_any()),
+                    Err(e) if !self.allow_partial_err(&e) => return Err(e),
+                    Ok(None) | Err(_) => return Ok(PyList::empty(py).into_any()),
                 };
 
                 let mut vec: SmallVec<[Bound<'_, PyAny>; 8]> = SmallVec::with_capacity(8);
-                if let Err(e) = self._parse_array(py, peek_first, &mut vec) {
-                    if !self._allow_partial_err(&e) {
+                if let Err(e) = self.parse_array(py, peek_first, &mut vec) {
+                    if !self.allow_partial_err(&e) {
                         return Err(e);
                     }
                 }
 
-                Ok(PyList::new_bound(py, vec).into_any())
+                Ok(PyList::new(py, vec)
+                    .map_err(|e| py_err_to_json_err(&e, self.parser.index))?
+                    .into_any())
             }
             Peek::Object => {
-                let dict = PyDict::new_bound(py);
-                if let Err(e) = self._parse_object(py, &dict) {
-                    if !self._allow_partial_err(&e) {
+                let dict = PyDict::new(py);
+                if let Err(e) = self.parse_object(py, &dict) {
+                    if !self.allow_partial_err(&e) {
                         return Err(e);
                     }
                 }
@@ -170,22 +171,22 @@ impl<'j, StringCache: StringMaybeCache, KeyCheck: MaybeKeyCheck, ParseNumber: Ma
         }
     }
 
-    fn _parse_array<'py>(
+    fn parse_array<'py>(
         &mut self,
         py: Python<'py>,
         peek_first: Peek,
         vec: &mut SmallVec<[Bound<'py, PyAny>; 8]>,
     ) -> JsonResult<()> {
-        let v = self._check_take_value(py, peek_first)?;
+        let v = self.check_take_value(py, peek_first)?;
         vec.push(v);
         while let Some(peek) = self.parser.array_step()? {
-            let v = self._check_take_value(py, peek)?;
+            let v = self.check_take_value(py, peek)?;
             vec.push(v);
         }
         Ok(())
     }
 
-    fn _parse_object<'py>(&mut self, py: Python<'py>, dict: &Bound<'py, PyDict>) -> JsonResult<()> {
+    fn parse_object<'py>(&mut self, py: Python<'py>, dict: &Bound<'py, PyDict>) -> JsonResult<()> {
         let set_item = |key: Bound<'py, PyString>, value: Bound<'py, PyAny>| {
             let r = unsafe { ffi::PyDict_SetItem(dict.as_ptr(), key.as_ptr(), value.as_ptr()) };
             // AFAIK this shouldn't happen since the key will always be a string  which is hashable
@@ -197,23 +198,23 @@ impl<'j, StringCache: StringMaybeCache, KeyCheck: MaybeKeyCheck, ParseNumber: Ma
         if let Some(first_key) = self.parser.object_first::<StringDecoder>(&mut self.tape)? {
             let first_key_s = first_key.as_str();
             check_keys.check(first_key_s, self.parser.index)?;
-            let first_key = StringCache::get_key(py, first_key_s, first_key.ascii_only());
+            let first_key = StringCache::get_key(py, first_key);
             let peek = self.parser.peek()?;
-            let first_value = self._check_take_value(py, peek)?;
+            let first_value = self.check_take_value(py, peek)?;
             set_item(first_key, first_value);
             while let Some(key) = self.parser.object_step::<StringDecoder>(&mut self.tape)? {
                 let key_s = key.as_str();
                 check_keys.check(key_s, self.parser.index)?;
-                let key = StringCache::get_key(py, key_s, key.ascii_only());
+                let key = StringCache::get_key(py, key);
                 let peek = self.parser.peek()?;
-                let value = self._check_take_value(py, peek)?;
+                let value = self.check_take_value(py, peek)?;
                 set_item(key, value);
             }
         }
         Ok(())
     }
 
-    fn _allow_partial_err(&self, e: &JsonError) -> bool {
+    fn allow_partial_err(&self, e: &JsonError) -> bool {
         if self.partial_mode.is_active() {
             e.allowed_if_partial()
         } else {
@@ -221,7 +222,7 @@ impl<'j, StringCache: StringMaybeCache, KeyCheck: MaybeKeyCheck, ParseNumber: Ma
         }
     }
 
-    fn _check_take_value<'py>(&mut self, py: Python<'py>, peek: Peek) -> JsonResult<Bound<'py, PyAny>> {
+    fn check_take_value<'py>(&mut self, py: Python<'py>, peek: Peek) -> JsonResult<Bound<'py, PyAny>> {
         self.recursion_limit = match self.recursion_limit.checked_sub(1) {
             Some(limit) => limit,
             None => return json_err!(RecursionLimitExceeded, self.parser.index),
@@ -238,7 +239,7 @@ const PARTIAL_ERROR: &str = "Invalid partial mode, should be `'off'`, `'on'`, `'
 
 impl<'py> FromPyObject<'py> for PartialMode {
     fn extract_bound(ob: &Bound<'py, PyAny>) -> PyResult<Self> {
-        if let Ok(bool_mode) = ob.downcast::<PyBool>() {
+        if let Ok(bool_mode) = ob.cast::<PyBool>() {
             Ok(bool_mode.is_true().into())
         } else if let Ok(str_mode) = ob.extract::<&str>() {
             match str_mode {
@@ -298,7 +299,10 @@ impl MaybeParseNumber for ParseNumberLossy {
         allow_inf_nan: bool,
     ) -> JsonResult<Bound<'py, PyAny>> {
         match parser.consume_number::<NumberAny>(peek.into_inner(), allow_inf_nan) {
-            Ok(number) => Ok(number.to_object(py).into_bound(py)),
+            Ok(number) => Ok(number
+                .into_pyobject(py)
+                .map_err(|e| py_err_to_json_err(&e, parser.index))?
+                .into_any()),
             Err(e) => {
                 if !peek.is_num() {
                     Err(json_error!(ExpectedSomeValue, parser.index))
@@ -325,11 +329,15 @@ impl MaybeParseNumber for ParseNumberLossless {
                 let obj = if number_range.is_int {
                     NumberAny::decode(bytes, 0, peek.into_inner(), allow_inf_nan)?
                         .0
-                        .to_object(py)
+                        .into_pyobject(py)
+                        .map_err(|e| py_err_to_json_err(&e, parser.index))?
                 } else {
-                    LosslessFloat::new_unchecked(bytes.to_vec()).into_py(py)
+                    LosslessFloat::new_unchecked(bytes.to_vec())
+                        .into_pyobject(py)
+                        .map_err(|e| py_err_to_json_err(&e, parser.index))?
+                        .into_any()
                 };
-                Ok(obj.into_bound(py))
+                Ok(obj)
             }
             Err(e) => {
                 if !peek.is_num() {
@@ -357,17 +365,17 @@ impl MaybeParseNumber for ParseNumberDecimal {
                 if number_range.is_int {
                     let obj = NumberAny::decode(bytes, 0, peek.into_inner(), allow_inf_nan)?
                         .0
-                        .to_object(py);
-                    Ok(obj.into_bound(py))
+                        .into_pyobject(py)
+                        .map_err(|e| py_err_to_json_err(&e, parser.index))?;
+                    Ok(obj.into_any())
                 } else {
-                    let decimal_type = get_decimal_type(py)
-                        .map_err(|e| JsonError::new(JsonErrorType::InternalError(e.to_string()), parser.index))?;
+                    let decimal_type = get_decimal_type(py).map_err(|e| py_err_to_json_err(&e, parser.index))?;
                     // SAFETY: NumberRange::decode has already confirmed that bytes are a valid JSON number,
                     // and therefore valid str
                     let float_str = unsafe { std::str::from_utf8_unchecked(bytes) };
                     decimal_type
                         .call1((float_str,))
-                        .map_err(|e| JsonError::new(JsonErrorType::InternalError(e.to_string()), parser.index))
+                        .map_err(|e| py_err_to_json_err(&e, parser.index))
                 }
             }
             Err(e) => {
@@ -380,3 +388,7 @@ impl MaybeParseNumber for ParseNumberDecimal {
         }
     }
 }
+
+fn py_err_to_json_err(e: &PyErr, index: usize) -> JsonError {
+    JsonError::new(JsonErrorType::InternalError(e.to_string()), index)
+}
